@@ -1,7 +1,6 @@
 #!/usr/bin/env python2
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -13,6 +12,7 @@ from lxml.etree import tostring
 
 from calibre.ebooks.metadata.toc import TOC
 from calibre.ebooks.oeb.polish.toc import elem_to_toc_text
+from polyglot.builtins import iteritems, range
 
 
 def from_headings(body, log, namespace):
@@ -21,10 +21,10 @@ def from_headings(body, log, namespace):
     headings = ('h1', 'h2', 'h3')
     tocroot = TOC()
     xpaths = [XPath('//%s' % x) for x in headings]
-    level_prev = {i+1:None for i in xrange(len(xpaths))}
+    level_prev = {i+1:None for i in range(len(xpaths))}
     level_prev[0] = tocroot
     level_item_map = {i+1:frozenset(xp(body)) for i, xp in enumerate(xpaths)}
-    item_level_map = {e:i for i, elems in level_item_map.iteritems() for e in elems}
+    item_level_map = {e:i for i, elems in iteritems(level_item_map) for e in elems}
 
     idcount = count()
 
@@ -48,7 +48,7 @@ def from_headings(body, log, namespace):
         text = elem_to_toc_text(item)
         toc = parent.add_item('index.html', elem_id, text)
         level_prev[lvl] = toc
-        for i in xrange(lvl+1, len(xpaths)+1):
+        for i in range(lvl+1, len(xpaths)+1):
             level_prev[i] = None
 
     if len(tuple(tocroot.flat())) > 1:
@@ -78,7 +78,7 @@ def structure_toc(entries):
         parent = find_parent(level)
         last_found[level] = parent.add_item('index.html', item.anchor,
                     item.text)
-        for i in xrange(level+1, len(last_found)):
+        for i in range(level+1, len(last_found)):
             last_found[i] = None
 
     return newtoc
@@ -93,7 +93,7 @@ def link_to_txt(a, styles, object_map):
                 if rs.css.get('display', None) == 'none':
                     a.remove(child)
 
-    return tostring(a, method='text', with_tail=False, encoding=unicode).strip()
+    return tostring(a, method='text', with_tail=False, encoding='unicode').strip()
 
 
 def from_toc(docx, link_map, styles, object_map, log, namespace):
